@@ -30,6 +30,7 @@
 
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
+#include <glib-unix.h>
 #include <gtk/gtk.h>
 #include <libnotify/notify.h>
 
@@ -44,7 +45,6 @@
 static gboolean   replace      = FALSE;
 static gboolean   debug        = FALSE;
 static gboolean   do_timed_exit = FALSE;
-static int        term_signal_pipe_fds[2];
 static guint      name_id      = 0;
 static GnomeSettingsManager *manager = NULL;
 
@@ -63,19 +63,6 @@ timed_exit_cb (void)
         return FALSE;
 }
 
-static void
-on_session_over (GDBusProxy *proxy,
-                 gchar      *sender_name,
-                 gchar      *signal_name,
-                 GVariant   *parameters,
-                 gpointer    user_data)
-{
-        if (g_strcmp0 (signal_name, "SessionOver") == 0) {
-                g_debug ("Got a SessionOver signal - stopping");
-                gtk_main_quit ();
-        }
-}
-
 static void
 respond_to_end_session (GDBusProxy *proxy)
 {
@@ -100,9 +87,6 @@ client_proxy_signal_cb (GDBusProxy *proxy,
         } else if (g_strcmp0 (signal_name, "EndSession") == 0) {
                 g_debug ("Got EndSession signal");
                 respond_to_end_session (proxy);
-        } else if (g_strcmp0 (signal_name, "Stop") == 0) {
-                g_debug ("Got Stop signal");
-                gtk_main_quit ();
         }
 }
 
@@ -277,8 +261,6 @@ register_with_gnome_session (GDBusProxy *proxy)
 {
         const char *startup_id;
 
-        g_signal_connect (G_OBJECT (proxy), "g-signal",
-                          G_CALLBACK (on_session_over), NULL);
         startup_id = g_getenv ("DESKTOP_AUTOSTART_ID");
         g_dbus_proxy_call (proxy,
                            "RegisterClient",
@@ -291,12 +273,8 @@ register_with_gnome_session (GDBusProxy *proxy)
 }
 
 static gboolean
-on_term_signal_pipe_closed (GIOChannel *source,
-                            GIOCondition condition,
-                            gpointer data)
+on_term_signal (gpointer user_data)
 {
-        term_signal_pipe_fds[0] = -1;
-
         g_debug ("Received SIGTERM - shutting down");
         /* Got SIGTERM, time to clean up and get out
          */
@@ -305,35 +283,6 @@ on_term_signal_pipe_closed (GIOChannel *source,
         return FALSE;
 }
 
-static void
-on_term_signal (int signal)
-{
-        /* Wake up main loop to tell it to shutdown */
-        close (term_signal_pipe_fds[1]);
-        term_signal_pipe_fds[1] = -1;
-}
-
-static void
-watch_for_term_signal (GnomeSettingsManager *manager)
-{
-        GIOChannel *channel;
-
-        if (-1 == pipe (term_signal_pipe_fds) ||
-            -1 == fcntl (term_signal_pipe_fds[0], F_SETFD, FD_CLOEXEC) ||
-            -1 == fcntl (term_signal_pipe_fds[1], F_SETFD, FD_CLOEXEC)) {
-                g_error ("Could not create pipe: %s", g_strerror (errno));
-                exit (EXIT_FAILURE);
-        }
-
-        channel = g_io_channel_unix_new (term_signal_pipe_fds[0]);
-        g_io_channel_set_encoding (channel, NULL, NULL);
-        g_io_channel_set_buffered (channel, FALSE);
-        g_io_add_watch (channel, G_IO_HUP, on_term_signal_pipe_closed, manager);
-        g_io_channel_unref (channel);
-
-        signal (SIGTERM, on_term_signal);
-}
-
 static void
 name_acquired_handler (GDBusConnection *connection,
                        const gchar *name,
@@ -350,7 +299,7 @@ name_acquired_handler (GDBusConnection *connection,
 #endif
         start_settings_manager ();
         register_with_gnome_session (proxy);
-        watch_for_term_signal (manager);
+        g_unix_signal_add (SIGTERM, on_term_signal, manager);
 }
 
 static void
