@@ -205,6 +205,10 @@ engine_properties_changed (GsdRfkillManager *manager)
         GVariantBuilder props_builder;
         GVariant *props_changed = NULL;
 
+        /* not yet connected to the session bus */
+        if (manager->priv->connection == NULL)
+                return;
+
         g_variant_builder_init (&props_builder, G_VARIANT_TYPE ("a{sv}"));
 
         g_variant_builder_add (&props_builder, "{sv}", "AirplaneMode",
@@ -271,10 +275,6 @@ rfkill_changed (CcRfkillGlib     *rfkill,
                 }
 	}
 
-        /* not yet connected to the bus */
-        if (manager->priv->connection == NULL)
-                return;
-
         engine_properties_changed (manager);
 }
 
@@ -657,8 +657,10 @@ gsd_rfkill_manager_stop (GsdRfkillManager *manager)
 
         g_debug ("Stopping rfkill manager");
 
-        if (manager->priv->name_id != 0)
+        if (manager->priv->name_id != 0) {
                 g_bus_unown_name (manager->priv->name_id);
+                manager->priv->name_id = 0;
+        }
 
         g_clear_pointer (&p->introspection_data, g_dbus_node_info_unref);
         g_clear_object (&p->connection);
