@@ -37,6 +37,12 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+
+#include <libgnome-desktop/gnome-rr-config.h>
+#include <libgnome-desktop/gnome-rr.h>
+#include <libgnome-desktop/gnome-pnp-ids.h>
+
 #include "gnome-settings-profile.h"
 #include "gsd-enums.h"
 #include "gsd-xsettings-manager.h"
@@ -221,8 +227,17 @@
  */
 #define DPI_FALLBACK 96
 
+/* The minimum resolution at which we turn on a window-scale of 2 */
 #define HIDPI_LIMIT (DPI_FALLBACK * 2)
 
+/* The minimum screen height at which we turn on a window-scale of 2;
+ * below this there just isn't enough vertical real estate for GNOME
+ * apps to work, and it's better to just be tiny */
+#define HIDPI_MIN_HEIGHT 1200
+
+/* From http://en.wikipedia.org/wiki/4K_resolution#Resolutions_of_common_formats */
+#define SMALLEST_4K_WIDTH 3656
+
 typedef struct _TranslationEntry TranslationEntry;
 typedef void (* TranslationFunc) (GnomeXSettingsManager *manager,
                                   TranslationEntry      *trans,
@@ -257,6 +272,8 @@ struct GnomeXSettingsManagerPrivate
 
         GsdRemoteDisplayManager *remote_display;
 
+        GnomeRRScreen     *rr_screen;
+
         guint              shell_name_watch_id;
         gboolean           have_shell;
 
@@ -426,33 +443,148 @@ get_dpi_from_gsettings (GnomeXSettingsManager *manager)
         return dpi * factor;
 }
 
+static GnomeRROutput *
+get_primary_output (GnomeRRScreen *screen)
+{
+        GnomeRROutput *primary = NULL;
+        GnomeRROutput **outputs;
+        guint i;
+
+        outputs = gnome_rr_screen_list_outputs (screen);
+        if (outputs == NULL || outputs[0] == NULL)
+                return NULL;
+        for (i = 0; outputs[i] != NULL; i++) {
+                if (gnome_rr_output_get_is_primary (outputs[i])) {
+                        primary = outputs[i];
+                        break;
+                }
+        }
+        if (primary == NULL)
+                primary = outputs[0];
+
+        return primary;
+}
+
+static gboolean
+primary_monitor_is_4k (GnomeRROutput *primary)
+{
+        GnomeRRMode *mode;
+
+        mode = gnome_rr_output_get_current_mode (primary);
+        if (gnome_rr_mode_get_width (mode) >= SMALLEST_4K_WIDTH)
+                return TRUE;
+        return FALSE;
+}
+
+static gboolean
+primary_monitor_on_hdmi (GnomeRROutput *primary)
+{
+        const char *name;
+
+        name = gnome_rr_output_get_name (primary);
+        if (name == NULL ||
+            strstr (name, "HDMI") == NULL)
+                return FALSE;
+        return TRUE;
+}
+
+static gboolean
+primary_monitor_should_skip_resolution_check (GnomeRROutput *primary)
+{
+        if (!primary_monitor_is_4k (primary) && primary_monitor_on_hdmi (primary))
+                return TRUE;
+
+        return FALSE;
+}
+
+static void
+get_dimensions_xrandr (GnomeRROutput *primary,
+                       int           *width,
+                       int           *height,
+                       int           *width_mm,
+                       int           *height_mm)
+{
+        GnomeRRMode *mode;
+
+        mode = gnome_rr_output_get_current_mode (primary);
+        *width = gnome_rr_mode_get_width (mode);
+        *height = gnome_rr_mode_get_height (mode);
+
+        gnome_rr_output_get_physical_size (primary,
+                                           width_mm,
+                                           height_mm);
+}
+
+static void
+get_dimensions_gdk (int *width,
+                    int *height,
+                    int *width_mm,
+                    int *height_mm)
+{
+        GdkDisplay *display;
+        GdkScreen *screen;
+        GdkRectangle rect;
+        int primary;
+        int monitor_scale;
+
+        display = gdk_display_get_default ();
+        screen = gdk_display_get_default_screen (display);
+        primary = gdk_screen_get_primary_monitor (screen);
+        gdk_screen_get_monitor_geometry (screen, primary, &rect);
+        monitor_scale = gdk_screen_get_monitor_scale_factor (screen, primary);
+
+        *width = rect.width * monitor_scale;
+        *height = rect.height * monitor_scale;
+
+        *width_mm = gdk_screen_get_monitor_width_mm (screen, primary);
+        *height_mm = gdk_screen_get_monitor_height_mm (screen, primary);
+}
+
 static int
 get_window_scale (GnomeXSettingsManager *manager)
 {
 	GSettings  *interface_settings;
         int window_scale;
-        GdkRectangle rect;
-        GdkDisplay *display;
-        GdkScreen *screen;
+        int width, height;
         int width_mm, height_mm;
-        int monitor_scale;
         double dpi_x, dpi_y;
 
 	interface_settings = g_hash_table_lookup (manager->priv->settings, INTERFACE_SETTINGS_SCHEMA);
         window_scale =
                 g_settings_get_uint (interface_settings, SCALING_FACTOR_KEY);
         if (window_scale == 0) {
-                display = gdk_display_get_default ();
-                screen = gdk_display_get_default_screen (display);
-                gdk_screen_get_monitor_geometry (screen, 0, &rect);
-                width_mm = gdk_screen_get_monitor_width_mm (screen, 0);
-                height_mm = gdk_screen_get_monitor_height_mm (screen, 0);
-                monitor_scale = gdk_screen_get_monitor_scale_factor (screen, 0);
+                GnomeRROutput *output = NULL;
+
+                window_scale = 1;
+
+                if (manager->priv->rr_screen)
+                        output = get_primary_output (manager->priv->rr_screen);
+
+                if (output) {
+                        if (primary_monitor_should_skip_resolution_check (output))
+                                goto out;
+
+                        get_dimensions_xrandr (output,
+                                               &width, &height,
+                                               &width_mm, &height_mm);
+                } else {
+                        /* Before the D-Bus DisplayConfig service exported by
+                         * Mutter becomes available, use the current information
+                         * that GDK has from the X server; in simple cases, this
+                         * will hopefully keep us from switching the window_scale
+                         * during startup.
+                         */
+                        get_dimensions_gdk (&width, &height,
+                                            &width_mm, &height_mm);
+                }
+
+                if (height < HIDPI_MIN_HEIGHT)
+                        goto out;
 
                 window_scale = 1;
                 if (width_mm > 0 && height_mm > 0) {
-                        dpi_x = (double)rect.width * monitor_scale / (width_mm / 25.4);
-                        dpi_y = (double)rect.height * monitor_scale / (height_mm / 25.4);
+                        dpi_x = (double)width / (width_mm / 25.4);
+                        dpi_y = (double)height / (height_mm / 25.4);
                         /* We don't completely trust these values so both
                            must be high, and never pick higher ratio than
                            2 automatically */
@@ -461,6 +593,7 @@ get_window_scale (GnomeXSettingsManager *manager)
                 }
         }
 
+out:
         return window_scale;
 }
 
@@ -969,6 +1102,33 @@ force_disable_animation_changed (GObject    *gobject,
         }
 }
 
+static void
+on_rr_screen_changed (GnomeRRScreen         *screen,
+                      GnomeXSettingsManager *manager)
+{
+        update_xft_settings (manager);
+        queue_notify (manager);
+}
+
+static void
+on_rr_screen_acquired (GObject      *object,
+                       GAsyncResult *result,
+                       gpointer      data)
+{
+        GnomeXSettingsManager *manager = data;
+        GnomeRRScreen *rr_screen;
+
+        rr_screen = gnome_rr_screen_new_finish (result, NULL);
+        if (!rr_screen)
+                return;
+
+        manager->priv->rr_screen = rr_screen;
+        g_signal_connect (rr_screen, "changed",
+                          G_CALLBACK (on_rr_screen_changed), manager);
+
+        on_rr_screen_changed (rr_screen, manager);
+}
+
 gboolean
 gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                                GError               **error)
@@ -991,6 +1151,10 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
         g_signal_connect (G_OBJECT (manager->priv->remote_display), "notify::force-disable-animations",
                           G_CALLBACK (force_disable_animation_changed), manager);
 
+        gnome_rr_screen_new_async (gdk_screen_get_default (),
+                                   on_rr_screen_acquired,
+                                   manager);
+
         manager->priv->settings = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                          NULL, (GDestroyNotify) g_object_unref);
 
@@ -1080,6 +1244,13 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
 
         g_clear_object (&manager->priv->remote_display);
 
+        if (manager->priv->rr_screen != NULL) {
+                g_signal_handlers_disconnect_by_func (manager->priv->rr_screen,
+                                                      (gpointer) on_rr_screen_changed,
+                                                      manager);
+                g_clear_object (&manager->priv->rr_screen);
+        }
+
         if (p->shell_name_watch_id > 0)
                 g_bus_unwatch_name (p->shell_name_watch_id);
 
